@@ -310,7 +310,7 @@ NewFile.createMdiChild = function(fileName, nameFilter, uiFile, graphicsSceneCla
         for (k=0; k<NewFile.postNewActions.length; k++) {
             if (!isNull(NewFile.postNewActions[k])) {
                 include(NewFile.postNewActions[k]);
-                if (isFunction(initNewFile)) {
+                if (typeof(initNewFile)!=="undefined" && isFunction(initNewFile)) {
                     initNewFile(mdiChild);
                 }
                 action = RGuiAction.getByScriptFile(NewFile.postNewActions[k]);
@@ -342,7 +342,8 @@ NewFile.updateTitle = function(mdiChild) {
 
     var document = mdiChild.getDocument();
     var fileName = document.getFileName();
-    var title = undefined;
+    var title = "";
+    var titleAppWin = "";
 
     // untitled:
     if (fileName==="") {
@@ -355,15 +356,17 @@ NewFile.updateTitle = function(mdiChild) {
             documentCounter++;
             mdiChild.setWindowTitle(title);
         }
+        titleAppWin = mdiChild.windowTitle;
+        fileName = "Untitled.dxf";
     }
     else {
         var fi = new QFileInfo(fileName);
         var name = fi.fileName();
-        var roStr = qsTr("read-only");
         if (fi.isWritable()) {
             title = addDirtyFlag(name);
         }
         else {
+            var roStr = qsTr("read-only");
             title = name + " " + roStr;
         }
 
@@ -371,11 +374,20 @@ NewFile.updateTitle = function(mdiChild) {
         if (!isNull(tabBar)) {
             tabBar.setTabToolTip(tabBar.currentIndex, fileName);
         }
+        titleAppWin = title;
+        title = title.replace(/&/g, "&&");
         mdiChild.setWindowTitle(title);
     }
 
+    // TODO: implement proxy icon with drag and drop
+    // triggers QIconDragEvent
+    //if (RS.getSystemId()==="osx") {
+        //appWin.windowFilePath = fileName;
+        //appWin.windowModified = document.isModified();
+    //}
+
     appWin.setWindowTitle(
-        stripDirtyFlag(mdiChild.windowTitle) +
+        stripDirtyFlag(titleAppWin) +
         (document.isModified() ? " *" : "") +
         " - " + qApp.applicationName
     );
