@@ -1888,7 +1888,7 @@ String.prototype.wordWrap = function(m, c, b){
 String.prototype.regexIndexOf = function(regex, startpos) {
     var indexOf = this.substring(startpos || 0).search(regex);
     return (indexOf >= 0) ? (indexOf + (startpos || 0)) : indexOf;
-}
+};
 
 String.prototype.regexLastIndexOf = function(regex, startpos) {
     regex = (regex.global) ? regex : new RegExp(regex.source, "g" + (regex.ignoreCase ? "i" : "") + (regex.multiLine ? "m" : ""));
@@ -1905,7 +1905,7 @@ String.prototype.regexLastIndexOf = function(regex, startpos) {
         regex.lastIndex = ++nextStop;
     }
     return lastIndexOf;
-}
+};
 
 /**
  * Modulo which works also for negative numbers (workaround for
@@ -2367,6 +2367,7 @@ function initUserShortcuts() {
         if (isNull(scStringList)) {
             if (match) qDebug("explicitly no shortcuts");
             action.setShortcuts([]);
+            action.removeShortcuts();
             continue;
         }
 
@@ -2380,6 +2381,7 @@ function initUserShortcuts() {
             var sc = new QKeySequence(scStringList[k]);
             scList.push(sc);
         }
+        action.removeShortcuts();
         action.setShortcuts(scList);
     }
 
@@ -2410,6 +2412,7 @@ function addActionsToWidgets() {
 
     // delete previous shortcut delegates:
     // part of workaround for QTBUG-38256, QTBUG-57990:
+    /*
     if ((RSettings.getQtVersion()<0x050500 && RSettings.getQtVersion()>=0x050000) ||
         (RSettings.getQtVersion()===0x050800 && RS.getSystemId()==="linux")) {
 
@@ -2424,9 +2427,10 @@ function addActionsToWidgets() {
         }
         appWin.setProperty("DelegatedShortcutsObjs", []);
     }
+    */
 
     var actions = RGuiAction.getActions();
-    var widgetTypes = ["Menu", "ToolBar", "MatrixPanel", "Panel"];
+    var widgetTypes = ["Menu", "ToolBar", "MatrixPanel", "Panel", "UserToolBar1", "UserToolBar2"];
     for (var c=0; c<actions.length; ++c) {
         var a = actions[c];
         if (a.icon.isNull() && !a.isIconDisabled()) {
@@ -2444,6 +2448,7 @@ function addActionsToWidgets() {
 
             var visibility = true;
             if (wn[0]==="!") {
+                // action not visible by default in this widget:
                 visibility = false;
                 wn = wn.substring(1);
             }
@@ -2472,6 +2477,7 @@ function addActionsToWidgets() {
 
                 // workaround for QTBUG-38256 (action not triggered for letter based shortcuts in SUB menus):
                 // workaround for QTBUG-57990 (action not triggered for letter based shortcuts in ALL menus):
+                /*
                 if ((RSettings.getQtVersion()<0x050500 && RSettings.getQtVersion()>=0x050000) ||
                     (RSettings.getQtVersion()===0x050800 && RS.getSystemId()==="linux")) {
 
@@ -2502,6 +2508,7 @@ function addActionsToWidgets() {
                         a.setDefaultShortcuts([]);
                     }
                 }
+                */
 
                 if (visibility) {
                     RGuiAction.addToWidget(a, w);
@@ -2522,7 +2529,9 @@ function addActionsToWidgets() {
             }
         }
 
-        if (!addedToWidget) {
+        if ((RSettings.getQtVersion()<0x050500 && RSettings.getQtVersion()>=0x050000) ||
+            (RSettings.getQtVersion()===0x050800 && RS.getSystemId()==="linux") ||
+            !addedToWidget) {
             // always add action to main window to make sure keycodes
             // are active even if action is invisible:
             RGuiAction.addToWidget(a, appWin);
@@ -2646,15 +2655,15 @@ function autoIconPath(path) {
 }
 
 function applyTheme() {
+    var systemId = RS.getSystemId();
     var theme = RSettings.getValue("Theme/ThemeName", undefined);
-    if (!isNull(theme)) {
+    if (systemId!=="osx" && !isNull(theme)) {
         var path = "themes/" + theme + "/";
 
         qApp.styleSheet = "";
 
         // load stylesheet.css, stylesheet_[win|macos|linux].css:
         var found = false;
-        var systemId = RS.getSystemId();
         if (systemId==="osx") systemId = "macos";
 
         var postfixes = ["", "_" + systemId];
@@ -2710,8 +2719,8 @@ function writeTextFile(fileName, str) {
         var textStream = new QTextStream(file);
         textStream.setCodec("UTF-8");
         textStream.writeString(str);
+        file.close();
     }
-    file.close();
 }
 
 function getKeyboardModifiers(event) {
